/*

Robot

Staffan Melin
2014-08-18
2014-08-31/Bluetooth

NewPing library
http://playground.arduino.cc/Code/NewPing

Notes
uh oh (http://arduino.cc/en/reference/servo):
On boards other than the Mega, use of the Servo library disables analogWrite() (PWM) functionality on pins 9 and 10, whether or not there is a Servo on those pins. 

*/

#include <Servo.h>
#include <NewPing.h>

// constants

// Pin 13 has an LED connected on most Arduino boards.
// led is ON if state = BT (Bluetooth)
int led = 13;

const int leftDirPin1 = 2;
const int leftDirPin2 = 3;
const int leftEnablePin = 6;

const int rightDirPin1 = 4;
const int rightDirPin2 = 5;
const int rightEnablePin = 11;

const int myServoPin = 10;

const int myPingTrigPin = 19;
const int myPingEchoPin = 18;
const int myPingMax = 200; // any obstacles over this cm away till not be noticed

const int kMotorSpeed = 200;

// Bluetooth control
// inspiration from http://blog.miguelgrinberg.com/post/building-an-arduino-robot-part-vi-remote-control

#define BT_RX_PIN 15 // A0
#define BT_TX_PIN 14 // A1

#include <SoftwareSerial.h>
SoftwareSerial BTSerial(BT_RX_PIN, BT_TX_PIN);

// global variables

Servo myServo;
NewPing myPing(myPingTrigPin, myPingEchoPin, myPingMax);

// autonomous or Bluetooth switch
const int autoPin = 16; // A2
const int kStateAuto = 0;
const int kStateBT = 1; // Bluetooth

int autoState = kStateAuto;

// run once

void setup() {                

	pinMode(led, OUTPUT);

	// momentary pushbutton that switches state between BT and AUTO
	pinMode(autoPin, INPUT);

	// Bluetooth
    	BTSerial.begin(9600);

	// distance meter
	pinMode(myPingTrigPin, INPUT);
	pinMode(myPingEchoPin, INPUT);

	// servo
	myServo.attach(myServoPin);
	myServo.write(90); // front

	// Motor Left
	pinMode(leftDirPin1, OUTPUT);
	pinMode(leftDirPin2, OUTPUT);
	pinMode(leftEnablePin, OUTPUT);

	// Motor Right
	pinMode(rightDirPin1, OUTPUT);
	pinMode(rightDirPin2, OUTPUT);
	pinMode(rightEnablePin, OUTPUT);

	moveForward();
}

// run continously

int mDistance;
int lDistance;
int rDistance;

void loop() {

	digitalWrite(led, autoState);

	if (digitalRead(autoPin) == HIGH)
	{
		if (autoState == kStateBT)
		{
			autoState = kStateAuto;
		} else {
			autoState = kStateBT;
		}
		delay(1000);
	}


	if (autoState == kStateBT) {
		BTControl();
	} else {
	
	
	
		mDistance = meanPing();
	
		// filter out zero values
		if (mDistance > 0)
		{
			// obstacle in front?
			if (mDistance < 25)
			{
				// digitalWrite(led, HIGH);
				moveStop();
	
				// look to the left and measure free distance
				myServo.write(10);
				delay(1000);
	
				lDistance = meanPing();
	
				// look to the right and measure free distance
				myServo.write(170);
				delay(1000);
	
				rDistance = meanPing();
	
				// center servo
				myServo.write(90);
				delay(1000);
	
				// decide on a new direction
				
				if (lDistance < 20 && rDistance < 20) { // blocked both left and right?
					moveBackward();
					delay(1000);
					if (random(2) == 0)
						moveTurnLeft();
					else
						moveTurnRight();
	
					delay(200);
	
				} else if (lDistance > rDistance) // is left more free?
				{
					moveTurnLeft();
					delay(200);
				} else { // is right more free
					moveTurnRight();
					delay(200);
				}
	
				//digitalWrite(led, LOW);
	
	
			} else {
				moveForward();
			}
	//		delay(50);
		}
	}
}

void moveTurnRight(void)
{
	analogWrite(rightEnablePin, 0);

	// left motor forward
	analogWrite(leftEnablePin, kMotorSpeed);
	digitalWrite(leftDirPin1, LOW);
	digitalWrite(leftDirPin2, HIGH);
}

void moveTurnLeft(void)
{
	analogWrite(leftEnablePin, 0);

	// right motor forward
	analogWrite(rightEnablePin, kMotorSpeed);
	digitalWrite(rightDirPin1, LOW);
	digitalWrite(rightDirPin2, HIGH);
}

void moveForward(void)
{
	// left motor forward
	analogWrite(leftEnablePin, kMotorSpeed);
	digitalWrite(leftDirPin1, LOW);
	digitalWrite(leftDirPin2, HIGH);

	// right motor forward
	analogWrite(rightEnablePin, kMotorSpeed);
	digitalWrite(rightDirPin1, LOW);
	digitalWrite(rightDirPin2, HIGH);

}

void moveBackward(void)
{

	// left motor backward
	analogWrite(leftEnablePin, kMotorSpeed);
	digitalWrite(leftDirPin1, HIGH);
	digitalWrite(leftDirPin2, LOW);

	// right motor backward
	analogWrite(rightEnablePin, kMotorSpeed);
	digitalWrite(rightDirPin1, HIGH);
	digitalWrite(rightDirPin2, LOW);



}

void moveStop(void)
{
	// left motor backward
	analogWrite(leftEnablePin, 0);
	digitalWrite(leftDirPin1, LOW);
	digitalWrite(leftDirPin2, LOW);

	// right motor backward
	analogWrite(rightEnablePin, 0);
	digitalWrite(rightDirPin1, LOW);
	digitalWrite(rightDirPin2, LOW);


}

int meanPing()
{
	int i;
	int totalDistance = 0;

	for (i = 0; i < 3; i++)
	{
		totalDistance = totalDistance + myPing.ping_cm();
	}

	return (totalDistance / 3);
}



bool BTControl()
{
	bool retVal;

	if (BTSerial.available() <= 0)
	{
		//digitalWrite(led, 1);
		retVal = false;
	} else {
		retVal = true;
		digitalWrite(led, 0);
		char ch = BTSerial.read();
		switch (ch) {
	                case '8': // up
	                    moveForward();
	                    break;
	                case '2': // down
	                    moveBackward();
	                    break;
	                case '4': // left
	                    moveTurnLeft();
	                    break;
	                case '6': // right
	                    moveTurnRight();
	                    break;
	                case 'D': // release = stop
	                    moveStop();
	                    break;
	/*
	                case 'A': // function key #1
	                case 'C':
	                    cmd.key = command_t::keyF1;
	                    break;
	                case 'B': // function key #2
	                case 'E': // function key #3
	                    cmd.key = command_t::keyF3;
	                    break;
	                case 'F': // function key #4
	                    cmd.key = command_t::keyF4;
	                    break;
	*/ 
			default:
				break;
		} // switch
	}

	return (retVal);

}